/* ***** BEGIN LICENSE BLOCK *****
 *
 * CSparse: a Concise Sparse matrix package.
 * Copyright (c) 2006, Timothy A. Davis.
 * http://www.cise.ufl.edu/research/sparse/CSparse
 *
 * -------------------------------------------------------------------------
 *
 * CSparse is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * CSparse is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this Module; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ***** END LICENSE BLOCK ***** */

part of edu.emory.mathcs.csparse;

/**
 * Sparse matrix times dense vector.
 *
 * @author Piotr Wendykier (piotr.wendykier@gmail.com)
 *
 */
//public class Dcs_gaxpy {

/**
 * Sparse matrix times dense column vector, y = A*x+y.
 *
 * @param A
 *            column-compressed matrix
 * @param x
 *            size n, vector x
 * @param y
 *            size m, vector y
 * @return true if successful, false on error
 */
bool cs_gaxpy(Dcs A, Float64List x, Float64List y) {
    int p, j, n;
    Int32List Ap, Ai;
    Float64List Ax;
    if (!CS_CSC(A) || x == null || y == null)
        return (false); /* check inputs */
    n = A.n;
    Ap = A.p;
    Ai = A.i;
    Ax = A.x;
    for (j = 0; j < n; j++) {
        for (p = Ap[j]; p < Ap[j + 1]; p++) {
            y[Ai[p]] += Ax[p] * x[j];
        }
    }
    return (true);
}
